/** 
*  \file Physics.h
*  \brief contains definitions in the Physics namespace
*  \author R.K.Garcia <rayg@ssec.wisc.edu>
*
*  \version $Id: Physics.h,v 1.6 2004/10/07 15:48:25 maciek Exp $
*
*  \par Copyright:
*  \verbatim
*
*  Copyright UW/SSEC, ALL RIGHTS RESERVED, 2004
*  Space Science and Engineering Center
*  University of Wisconsin - Madison, USA
*
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version.
*
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*
*  \endverbatim
*/


#ifndef H_PHYSICS
#define H_PHYSICS

/**
*  \brief Physics namespace, including standard types and formulae for 
*  radiances, wavenumbers, temperatures, times, voltages.
*/
namespace Physics { 

/**
 * floating point time representation
 * Units: seconds
 */
typedef double Seconds_t;

/**
 * floating point temperature representation
 * Units: Kelvin
 */
typedef double Kelvin_t;

/**
 * floating point wavenumber representation
 * Units: wavenumbers
 */
typedef double Wnum_t;

/** inverse of wavenumbers - centimeters.
 */
typedef double Cm_t;

/** 
 * Radiance_t value
 * Units: 
 */
typedef double Radiance_t;


/** \fun planckRadiance
 * \param T temperature in Kelvin
 * \param wnum wavenumber
 * \todo Implement me!
 */
 
inline Radiance_t planckRadiance( Kelvin_t T, Wnum_t wnum )
{
    return Radiance_t( 0. ); 
}


/** physical constants needed for the relations implemented in this namespace */
struct Constants_t
{
    /** Planck constant. 
    * Units: #FIXME#
    */
    double planck;
};    

/** standardized view of regularly-spaced wavenumbers */
struct Wnums_t
{
    Wnum_t baseWnum;
    Wnum_t deltaWnum;
    unsigned wnumCount;
    
    /** truncate to bounds. FIXME: this should raise an exception if at all possible */
    inline unsigned trunc( unsigned offset ) const 
    { return (offset>=wnumCount)? wnumCount-1 : offset; }
    
    /** access as if it were an array */
    inline double operator[]( unsigned offset ) const
    { return deltaWnum*double(trunc(offset))+baseWnum ; }
};


/** \brief Transforms a Wnums_t class into a blackbody radiance spectrum
 * Future: this can create a cache full of NANs internally, and return values from the cache
 * instead of recomputing the power every time an access occurs. 
 * Possibly as an alternate implementation of the same interface, e.g. CachingPlanckRadianceGenerator
 */
class PlanckRadianceGenerator
{
    const Constants_t &C;
    const Wnums_t &W;
    const Kelvin_t T;
    
public:
    PlanckRadianceGenerator( const Constants_t &_c, const Wnums_t &_w, Kelvin_t _t ): 
        C(_c), W(_w), T(_t)
    { }
    
    inline Radiance_t operator[]( unsigned offset ) const
    {
        return planckRadiance( T, W[offset] ); 
    }
};
    
}; // namespace Physics


#endif
